
% Program to reconstruct undersampled radial DCE MRI data using temporal and spatial Total Variation constraints

% Ref: G. Adluru, C.J. McGann, P.Speier, E.G. Kholmovski, A. Shaaban, E.V.R. DiBella.  
% “Acquisition and reconstruction of undersampled radial data for myocardial perfusion MRI.”  
% Journal of Magnetic Resonance Imaging (JMRI), 29:466-473, 2009.

function recon_data=stcr_oneSlice_oneCoil(k_space,DEBUG)

weight_fidelity=1;
weight_temporal=0.04; % regularization parameter 1
weight_spatial=0.006; % regularization parameter 2
beta_sqrd=0.00000001;
stepsize=0.5;
noi=120;

nt=size(k_space,3);

offset_period=4; % period after which offset angle repeats

disp('Pre-interpolating data to Cartesian grid ')
[reduced_k_space mask_k_space_sparse]=pre_interp(k_space,offset_period);
measuredImgDomain=zeros(size(reduced_k_space));
for i=1:nt
    measuredImgDomain(:,:,i)=fftshift(ifft2(reduced_k_space(:,:,i)));
end
temporal_term_update=zeros(size(measuredImgDomain));
img_est=measuredImgDomain; 
W_img_est=ifft2(fft2(img_est).*mask_k_space_sparse);



% gradient descent minimization
if(DEBUG)
   fidelityNorm=zeros(noi,1);
   temporalNorm=zeros(noi,1);
   spatialNorm=zeros(noi,1);
   total_cost=zeros(noi,1);
end
tic
for iter_no=1:noi
    iter_no
    
    fidelity_update=weight_fidelity*(measuredImgDomain-W_img_est);
    
    % Computing temp regul - TV    
    temp_a=diff(img_est,1,3);
    temp_b=temp_a./(sqrt(beta_sqrd+(abs(temp_a).^2)));    
    temp_c=diff(temp_b,1,3);
    
    temporal_term_update(:,:,1)=temp_b(:,:,1);
    temporal_term_update(:,:,2:end-1)=temp_c;
    temporal_term_update(:,:,end)=-temp_b(:,:,end);   
    
    temporal_term_update=temporal_term_update*weight_temporal;
    
    % Computing spatial regul - TV    
    spatial_term_update=TV_Spatial_2D(img_est,beta_sqrd)*weight_spatial;
    
    if(DEBUG)
        fidelityNorm(iter_no)=sum(sum(sum(abs(fidelity_update).^2)));
        temporalNorm(iter_no)=sum(sum(sum(abs(diff(img_est,1,3)))));
        
        % spatial norm:  (note matlab gradient() call not the same as diff
        trunc=size(img_est,1)-1;
        sx_norm=abs(diff(img_est,1,2));
        sx_norm=sx_norm(1:trunc,1:trunc,:);
        sy_norm=abs(diff(img_est,1,1));
        sy_norm=sy_norm(1:trunc,1:trunc,:);
        spatialNorm(iter_no)=sum(sum(sum(sqrt(abs(sx_norm).^2+abs(sy_norm).^2))));
        
        figure(99);clf; hold on;  
        plot(weight_fidelity*fidelityNorm(1:iter_no),'c*-')
        plot(weight_temporal*temporalNorm(1:iter_no),'kx-');
        plot(weight_spatial*spatialNorm(1:iter_no),'bx-');
        totalNorm=weight_temporal*temporalNorm(1:iter_no)+weight_fidelity*fidelityNorm(1:iter_no)+weight_spatial*spatialNorm(1:iter_no);
        plot(totalNorm)
        legend('fidelity (squared L2)','temporal (L1 norm)','spatial (L1 norm)',  'total')
        title('All curves are norm times weight ')      
        drawnow
    end
    
    
    img_est=img_est+stepsize*(fidelity_update+temporal_term_update+spatial_term_update);    
    W_img_est=ifft2(fft2(img_est).*mask_k_space_sparse);    
    
    if(DEBUG)   % write out recon every 30 iterations. 
        if mod(iter_no,30)==0
           sx=size(img_est,1);
           recon_data=zeros(sx/2,sx/2,size(img_est,3));
           for i=1:size(recon_data,3)
            recon_data(:,:,i)=rot90(abs(img_est(sx/4+1:3*sx/4,sx/4+1:3*sx/4,i)),-1);
           end
           outfile=strcat('test_iter',int2str(iter_no),'_F',int2str(1000*weight_fidelity),'_T',int2str(1000*weight_temporal),'_S',int2str(1000*weight_spatial))
           save(outfile,'recon_data', 'fidelityNorm', 'temporalNorm', 'spatialNorm', 'totalNorm','stepsize');
        end
    end
end
toc

% crop to remove 2x read direction oversampling
sx=size(img_est,1);
recon_data=zeros(sx/2,sx/2,size(img_est,3));
for i=1:size(recon_data,3)
    recon_data(:,:,i)=rot90(abs(img_est(sx/4+1:3*sx/4,sx/4+1:3*sx/4,i)),-1);
end

if(DEBUG)
    figure(100);clf; hold on;  
    subplot(2,2,1); plot(weight_fidelity*fidelityNorm,'c*-'); title('Fidelity norm')
    subplot(2,2,2); plot(weight_temporal*temporalNorm,'kx-');  title('Temporal norm')
    subplot(2,2,3); plot(spatialNorm,'bx-'); title('Spatial norm')
end


return;

